@@ -1,4 +1,4 @@
-{-# LANGUAGE ScopedTypeVariables, MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances, TypeSynonymInstances, UndecidableInstances, FlexibleContexts, ScopedTypeVariables, DeriveDataTypeable #-}
+{-# LANGUAGE ScopedTypeVariables, MultiParamTypeClasses, FunctionalDependencies, FlexibleInstances, TypeSynonymInstances, UndecidableInstances, FlexibleContexts, ScopedTypeVariables, DeriveDataTypeable, Rank2Types #-}
 module LLVM.Core.CodeGen(
     -- * Module creation
     newModule, newNamedModule, defineModule, createModule,
@@ -7,20 +7,22 @@ module LLVM.Core.CodeGen(
     Linkage(..),
     Visibility(..),
     -- * Function creation
-    Function, newFunction, newNamedFunction, defineFunction, createFunction, createNamedFunction,
+    Function, newFunction, newNamedFunction, defineFunction, createFunction, createNamedFunction, setFuncCallConv,
     addAttributes,
     FFI.Attribute(..),
-    externFunction,
+    externFunction, staticFunction,
     FunctionArgs, FunctionRet,
     TFunction,
     -- * Global variable creation
     Global, newGlobal, newNamedGlobal, defineGlobal, createGlobal, createNamedGlobal, TGlobal,
+    externGlobal, staticGlobal,
     -- * Values
     Value(..), ConstValue(..),
     IsConst(..), valueOf, value,
     zero, allOnes, undef,
     createString, createStringNul,
-    constVector, constArray,
+    withString, withStringNul,
+    constVector, constArray, constStruct, constPackedStruct,
     -- * Basic blocks
     BasicBlock(..), newBasicBlock, newNamedBasicBlock, defineBasicBlock, createBasicBlock, getCurrentBasicBlock,
     fromLabel, toLabel,
@@ -31,7 +33,8 @@ import Data.Typeable
 import Control.Monad(liftM, when)
 import Data.Int
 import Data.Word
-import Foreign.Ptr(minusPtr, nullPtr)
+import Foreign.StablePtr (StablePtr, castStablePtrToPtr)
+import Foreign.Ptr(minusPtr, nullPtr, castPtr, FunPtr, castFunPtrToPtr)
 import Foreign.Storable(sizeOf)
 import Data.TypeLevel hiding (Bool, Eq, (+), (==))
 import LLVM.Core.CodeGenMonad
@@ -101,19 +104,26 @@ instance IsConst Float  where constOf = constF
 instance IsConst Double where constOf = constF
 --instance IsConst FP128  where constOf = constF
 
+constOfPtr :: (IsType a) =>
+    a -> Ptr b -> ConstValue a
+constOfPtr proto p =
+    let ip = p `minusPtr` nullPtr
+        inttoptrC (ConstValue v) = ConstValue $ FFI.constIntToPtr v (typeRef proto)
+    in  if sizeOf p == 4 then
+            inttoptrC $ constOf (fromIntegral ip :: Word32)
+        else if sizeOf p == 8 then
+            inttoptrC $ constOf (fromIntegral ip :: Word64)
+        else
+            error "constOf Ptr: pointer size not 4 or 8"
+
 -- This instance doesn't belong here, but mutually recursive modules are painful.
 instance (IsType a) => IsConst (Ptr a) where
-    constOf p =
-        let ip = p `minusPtr` nullPtr
-            inttoptrC (ConstValue v) = ConstValue $ FFI.constIntToPtr v (typeRef (undefined :: Ptr a))
-        in  if sizeOf p == 4 then
-                inttoptrC $ constOf (fromIntegral ip :: Word32)
-            else if sizeOf p == 8 then
-                inttoptrC $ constOf (fromIntegral ip :: Word64)
-            else
-                error "constOf Ptr: pointer size not 4 or 8"
+    constOf p = constOfPtr p p
 
-instance (IsPrimitive a, IsConst a, Nat n) => IsConst (Vector n a) where
+instance IsConst (StablePtr a) where
+    constOf p = constOfPtr p (castStablePtrToPtr p)
+
+instance (IsPrimitive a, IsConst a, Pos n) => IsConst (Vector n a) where
     constOf (Vector xs) = constVector (map constOf xs)
 
 instance (IsConst a, IsSized a s, Nat n) => IsConst (Array n a) where
@@ -189,7 +199,7 @@ newFunction :: forall a . (IsFunction a)
 newFunction linkage = genMSym "fun" >>= newNamedFunction linkage
 
 -- | Define a function body.  The basic block returned by the function is the function entry point.
-defineFunction :: forall f g r . (FunctionArgs f g (CodeGenFunction r ()))
+defineFunction :: forall f g r . (FunctionArgs f g r)
                => Function f       -- ^ Function to define (created by 'newFunction').
                -> g                -- ^ Function body.
                -> CodeGenModule ()
@@ -203,7 +213,7 @@ defineFunction (Value fn) body = do
     return ()
 
 -- | Create a new function with the given body.
-createFunction :: (IsFunction f, FunctionArgs f g (CodeGenFunction r ()))
+createFunction :: (IsFunction f, FunctionArgs f g r)
                => Linkage
                -> g  -- ^ Function body.
                -> CodeGenModule (Function f)
@@ -213,7 +223,7 @@ createFunction linkage body = do
     return f
 
 -- | Create a new function with the given body.
-createNamedFunction :: (IsFunction f, FunctionArgs f g (CodeGenFunction r ()))
+createNamedFunction :: (IsFunction f, FunctionArgs f g r)
                => Linkage
 	       -> String
                -> g  -- ^ Function body.
@@ -223,19 +233,27 @@ createNamedFunction linkage name body = do
     defineFunction f body
     return f
 
+-- | Set the calling convention of a function. By default it is the
+-- C calling convention.
+setFuncCallConv :: Function a
+                -> FFI.CallingConvention
+                -> CodeGenModule ()
+setFuncCallConv (Value f) cc = do
+  liftIO $ FFI.setFunctionCallConv f (FFI.fromCallingConvention cc)
+  return ()
+
 -- | Add attributes to a value.  Beware, what attributes are allowed depends on
 -- what kind of value it is.
 addAttributes :: Value a -> Int -> [FFI.Attribute] -> CodeGenFunction r ()
 addAttributes (Value f) i as = do
     liftIO $ FFI.addInstrAttribute f (fromIntegral i) (sum $ map FFI.fromAttribute as)
 
--- XXX This is ugly, it must be possible to make it simpler
 -- Convert a function of type f = t1->t2->...-> IO r to
 -- g = Value t1 -> Value t2 -> ... CodeGenFunction r ()
 class FunctionArgs f g r | f -> g r, g r -> f where
-    apArgs :: Int -> FunctionRef -> g -> r
+    apArgs :: Int -> FunctionRef -> g -> FA r
 
-applyArgs :: (FunctionArgs f g r) => FunctionRef -> g -> r
+applyArgs :: (FunctionArgs f g r) => FunctionRef -> g -> FA r
 applyArgs = apArgs 0
 
 instance (FunctionArgs b b' r) => FunctionArgs (a -> b) (Value a -> b') r where
@@ -244,31 +262,32 @@ instance (FunctionArgs b b' r) => FunctionArgs (a -> b) (Value a -> b') r where
 -- XXX instances for all IsFirstClass functions,
 -- because Haskell can't deal with the context and the FD
 type FA a = CodeGenFunction a ()
-instance FunctionArgs (IO Float)        (FA Float)        (FA Float)        where apArgs _ _ g = g
-instance FunctionArgs (IO Double)       (FA Double)       (FA Double)       where apArgs _ _ g = g
-instance FunctionArgs (IO FP128)        (FA FP128)        (FA FP128)        where apArgs _ _ g = g
+instance FunctionArgs (IO Float)         (FA Float)         Float         where apArgs _ _ g = g
+instance FunctionArgs (IO Double)        (FA Double)        Double        where apArgs _ _ g = g
+instance FunctionArgs (IO FP128)         (FA FP128)         FP128         where apArgs _ _ g = g
 instance (Pos n) => 
-         FunctionArgs (IO (IntN n))     (FA (IntN n))     (FA (IntN n))     where apArgs _ _ g = g
+         FunctionArgs (IO (IntN n))      (FA (IntN n))      (IntN n)      where apArgs _ _ g = g
 instance (Pos n) =>
-         FunctionArgs (IO (WordN n))    (FA (WordN n))    (FA (WordN n))    where apArgs _ _ g = g
-instance FunctionArgs (IO Bool)         (FA Bool)         (FA Bool)         where apArgs _ _ g = g
-instance FunctionArgs (IO Int8)         (FA Int8)         (FA Int8)         where apArgs _ _ g = g
-instance FunctionArgs (IO Int16)        (FA Int16)        (FA Int16)        where apArgs _ _ g = g
-instance FunctionArgs (IO Int32)        (FA Int32)        (FA Int32)        where apArgs _ _ g = g
-instance FunctionArgs (IO Int64)        (FA Int64)        (FA Int64)        where apArgs _ _ g = g
-instance FunctionArgs (IO Word8)        (FA Word8)        (FA Word8)        where apArgs _ _ g = g
-instance FunctionArgs (IO Word16)       (FA Word16)       (FA Word16)       where apArgs _ _ g = g
-instance FunctionArgs (IO Word32)       (FA Word32)       (FA Word32)       where apArgs _ _ g = g
-instance FunctionArgs (IO Word64)       (FA Word64)       (FA Word64)       where apArgs _ _ g = g
-instance FunctionArgs (IO ())           (FA ())           (FA ())           where apArgs _ _ g = g
+         FunctionArgs (IO (WordN n))     (FA (WordN n))     (WordN n)     where apArgs _ _ g = g
+instance FunctionArgs (IO Bool)          (FA Bool)          Bool          where apArgs _ _ g = g
+instance FunctionArgs (IO Int8)          (FA Int8)          Int8          where apArgs _ _ g = g
+instance FunctionArgs (IO Int16)         (FA Int16)         Int16         where apArgs _ _ g = g
+instance FunctionArgs (IO Int32)         (FA Int32)         Int32         where apArgs _ _ g = g
+instance FunctionArgs (IO Int64)         (FA Int64)         Int64         where apArgs _ _ g = g
+instance FunctionArgs (IO Word8)         (FA Word8)         Word8         where apArgs _ _ g = g
+instance FunctionArgs (IO Word16)        (FA Word16)        Word16        where apArgs _ _ g = g
+instance FunctionArgs (IO Word32)        (FA Word32)        Word32        where apArgs _ _ g = g
+instance FunctionArgs (IO Word64)        (FA Word64)        Word64        where apArgs _ _ g = g
+instance FunctionArgs (IO ())            (FA ())            ()            where apArgs _ _ g = g
 instance (Pos n, IsPrimitive a) =>
-         FunctionArgs (IO (Vector n a)) (FA (Vector n a)) (FA (Vector n a)) where apArgs _ _ g = g
+         FunctionArgs (IO (Vector n a))  (FA (Vector n a))  (Vector n a)  where apArgs _ _ g = g
 instance (IsType a) => 
-         FunctionArgs (IO (Ptr a))      (FA (Ptr a))      (FA (Ptr a))      where apArgs _ _ g = g
+         FunctionArgs (IO (Ptr a))       (FA (Ptr a))       (Ptr a)       where apArgs _ _ g = g
+instance FunctionArgs (IO (StablePtr a)) (FA (StablePtr a)) (StablePtr a) where apArgs _ _ g = g
 
 -- |This class is just to simplify contexts.
-class (FunctionArgs (IO a) (CodeGenFunction a ()) (CodeGenFunction a ())) => FunctionRet a
-instance (FunctionArgs (IO a) (CodeGenFunction a ()) (CodeGenFunction a ())) => FunctionRet a
+class (FunctionArgs (IO a) (CodeGenFunction a ()) a) => FunctionRet a
+instance (FunctionArgs (IO a) (CodeGenFunction a ()) a) => FunctionRet a
 
 --------------------------------------
 
@@ -308,20 +327,60 @@ fromLabel (Value ptr) = BasicBlock (FFI.valueAsBasicBlock ptr)
 
 --------------------------------------
 
--- |Create a reference to an external function while code generating for a function.
+--- XXX: the functions in this section (and addGlobalMapping) don't actually use any
+-- Function state so should really be in the CodeGenModule monad
+
+-- | Create a reference to an external function while code generating for a function.
+-- If LLVM cannot resolve its name, then you may try 'staticFunction'.
 externFunction :: forall a r . (IsFunction a) => String -> CodeGenFunction r (Function a)
-externFunction name = do
+externFunction name = externCore name $ fmap (unValue :: Function a -> FFI.ValueRef) . newNamedFunction ExternalLinkage
+
+-- | As 'externFunction', but for 'Global's rather than 'Function's
+externGlobal :: forall a r . (IsType a) => Bool -> String -> CodeGenFunction r (Global a)
+externGlobal isConst name = externCore name $ fmap (unValue :: Global a -> FFI.ValueRef) . newNamedGlobal isConst ExternalLinkage
+
+externCore :: forall a r . String -> (String -> CodeGenModule FFI.ValueRef) -> CodeGenFunction r (Global a)
+externCore name act = do
     es <- getExterns
     case lookup name es of
         Just f -> return $ Value f
         Nothing -> do
-            let linkage = ExternalLinkage
-            modul <- getFunctionModule
-            let typ = typeRef (undefined :: a)
-            f <- liftIO $ U.addFunction modul linkage name typ
+            f <- liftCodeGenModule $ act name
             putExterns ((name, f) : es)
 	    return $ Value f
 
+{- |
+Make an external C function with a fixed address callable from LLVM code.
+This callback function can also be a Haskell function,
+that was imported like
+
+> foreign import ccall "&nextElement"
+>    nextElementFunPtr :: FunPtr (StablePtr (IORef [Word32]) -> IO Word32)
+
+See @examples\/List.hs@.
+
+When you only use 'externFunction', then LLVM cannot resolve the name.
+(However, I do not know why.)
+Thus 'staticFunction' manages a list of static functions.
+This list is automatically installed by 'ExecutionEngine.simpleFunction'
+and can be manually obtained by 'getGlobalMappings'
+and installed by 'ExecutionEngine.addGlobalMappings'.
+\"Installing\" means calling LLVM's @addGlobalMapping@ according to
+<http://old.nabble.com/jit-with-external-functions-td7769793.html>.
+-}
+staticFunction :: forall f r. (IsFunction f) => FunPtr f -> CodeGenFunction r (Function f)
+staticFunction func = liftCodeGenModule $ do
+    val <- newNamedFunction ExternalLinkage ""
+    addGlobalMapping (unValue (val :: Function f)) (castFunPtrToPtr func)
+    return val
+
+-- | As 'staticFunction', but for 'Global's rather than 'Function's
+staticGlobal :: forall a r. (IsType a) => Bool -> Ptr a -> CodeGenFunction r (Global a)
+staticGlobal isConst gbl = liftCodeGenModule $ do
+    val <- newNamedGlobal isConst ExternalLinkage ""
+    addGlobalMapping (unValue (val :: Global a)) (castPtr gbl)
+    return val
+
 --------------------------------------
 
 withCurrentBuilder :: (FFI.BuilderRef -> IO a) -> CodeGenFunction r a
@@ -378,12 +437,37 @@ type TFunction a = CodeGenModule (Function a)
 type TGlobal a = CodeGenModule (Global a)
 
 -- Special string creators
+{-# DEPRECATED createString "use withString instead" #-}
 createString :: String -> TGlobal (Array n Word8)
 createString s = string (length s) (U.constString s)
 
+{-# DEPRECATED createStringNul "use withStringNul instead" #-}
 createStringNul :: String -> TGlobal (Array n Word8)
 createStringNul s = string (length s + 1) (U.constStringNul s)
 
+withString ::
+   String ->
+   (forall n. (Nat n) => Global (Array n Word8) -> CodeGenModule a) ->
+   CodeGenModule a
+withString s act =
+   let n = length s
+   in  reifyIntegral n (\tn ->
+          do arr <- string n (U.constString s)
+             act (fixArraySize tn arr))
+
+withStringNul ::
+   String ->
+   (forall n. (Nat n) => Global (Array n Word8) -> CodeGenModule a) ->
+   CodeGenModule a
+withStringNul s act =
+   let n = length s + 1
+   in  reifyIntegral n (\tn ->
+          do arr <- string n (U.constStringNul s)
+             act (fixArraySize tn arr))
+
+fixArraySize :: n -> Global (Array n a) -> Global (Array n a)
+fixArraySize _ = id
+
 string :: Int -> FFI.ValueRef -> TGlobal (Array n Word8)
 string n s = do
     modul <- getModule
@@ -397,7 +481,7 @@ string n s = do
 --------------------------------------
 
 -- |Make a constant vector.  Replicates or truncates the list to get length /n/.
-constVector :: forall a n . (Nat n) => [ConstValue a] -> ConstValue (Vector n a)
+constVector :: forall a n . (Pos n) => [ConstValue a] -> ConstValue (Vector n a)
 constVector xs =
     ConstValue $ U.constVector (toNum (undefined :: n)) [ v | ConstValue v <- xs ]
 
@@ -405,3 +489,21 @@ constVector xs =
 constArray :: forall a n s . (IsSized a s, Nat n) => [ConstValue a] -> ConstValue (Array n a)
 constArray xs =
     ConstValue $ U.constArray (typeRef (undefined :: a)) (toNum (undefined :: n)) [ v | ConstValue v <- xs ]
+
+-- |Make a constant struct.
+constStruct :: (IsConstStruct c a) => c -> ConstValue (Struct a)
+constStruct struct =
+    ConstValue $ U.constStruct (constValueFieldsOf struct) False
+
+-- |Make a constant packed struct.
+constPackedStruct :: (IsConstStruct c a) => c -> ConstValue (PackedStruct a)
+constPackedStruct struct =
+    ConstValue $ U.constStruct (constValueFieldsOf struct) True
+
+class IsConstStruct c a | a -> c, c -> a where
+    constValueFieldsOf :: c -> [FFI.ValueRef]
+
+instance (IsConst a, IsConstStruct cs as) => IsConstStruct (ConstValue a, cs) (a, as) where
+    constValueFieldsOf (a, as) = unConstValue a : constValueFieldsOf as
+instance IsConstStruct () () where
+    constValueFieldsOf _ = []
